# Generated by roxygen2: do not edit by hand

S3method(augment,tab_pfn)
S3method(predict,tab_pfn)
S3method(predict,tabpfn.classifier.TabPFNClassifier)
S3method(predict,tabpfn.regressor.TabPFNRegressor)
S3method(print,control_tab_pfn)
S3method(print,tab_pfn)
S3method(tab_pfn,data.frame)
S3method(tab_pfn,default)
S3method(tab_pfn,formula)
S3method(tab_pfn,matrix)
S3method(tab_pfn,recipe)
export(augment)
export(control_tab_pfn)
export(is_tab_pfn_installed)
export(tab_pfn)
export(tabpfn_download_models)
export(tabpfn_list_versions)
import(rlang)
importFrom(generics,augment)
importFrom(stats,predict)
