#' @keywords internal
"_PACKAGE"

## usethis namespace: start
#' @import rlang
#' @importFrom stats predict

#' @importFrom generics augment
#' @export
generics::augment

## usethis namespace: end
NULL

utils::globalVariables(
  c(
    ".row_order",
    "data",
    "outcome",
    "quantile",
    "sample_num",
    "sample_prop",
    "size"
  )
)
