# control values

    Code
      ctrl
    Message
      control object for `tab_pfn()`
    Output
      
    Message
      non-default arguments:
      * `n_preprocessing_jobs`: 2
      * `device`: "cpu"
      * `ignore_pretraining_limits`: TRUE
      * `inference_precision`: "float32"
      * `fit_mode`: "low_memory"
      * `memory_saving_mode`: TRUE
      * `random_state`: 12345

---

    Code
      control_tab_pfn(random_state = 1)
    Message
      control object for `tab_pfn()`
    Output
      
    Message
      non-default arguments:
      * `random_state`: 1

