# classification models

    Code
      mod_df
    Message
      TabPFN Classification Model
    Output
      
    Message
      Training set
      i 20 data points
      i 2 predictors
      i class levels: "Class1" and "Class2"

---

    Code
      mod_f
    Message
      TabPFN Classification Model
    Output
      
    Message
      Training set
      i 20 data points
      i 2 predictors
      i class levels: "Class1" and "Class2"

---

    Code
      mod_mat
    Message
      TabPFN Classification Model
    Output
      
    Message
      Training set
      i 20 data points
      i 2 predictors
      i class levels: "Class1" and "Class2"

# classification models - recipes

    Code
      mod_rec
    Message
      TabPFN Classification Model
    Output
      
    Message
      Training set
      i 20 data points
      i 3 predictors
      i class levels: "Class1" and "Class2"

# main options

    `num_estimators` must be a whole number, not the string "YES".

---

    `softmax_temperature` must be a number larger than or equal to 2.22044604925031e-16, not the number -1.

---

    `balance_probabilities` must be a logical vector, not the string "nope".

---

    `average_before_softmax` must be a logical vector, not the string "suuuure".

