# regression models

    Code
      mod_df
    Message
      TabPFN Regression Model
    Output
      
    Message
      Training set
      i 32 data points
      i 10 predictors

---

    Code
      mod_f
    Message
      TabPFN Regression Model
    Output
      
    Message
      Training set
      i 32 data points
      i 10 predictors

---

    Code
      mod_mat
    Message
      TabPFN Regression Model
    Output
      
    Message
      Training set
      i 32 data points
      i 10 predictors

---

    `tab_pfn()` is not defined for the number 1.

# regression models - recipes

    Code
      mod_rec
    Message
      TabPFN Regression Model
    Output
      
    Message
      Training set
      i 20 data points
      i 5 predictors

