% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/misc.R
\name{is_tab_pfn_installed}
\alias{is_tab_pfn_installed}
\title{Check the Python package installation}
\usage{
is_tab_pfn_installed()
}
\value{
A single logical
}
\description{
Attempts to import the Python package
}
\examples{
if (interactive()) {
 # This may take a minute
 is_tab_pfn_installed()
}
}
