% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/control.R
\name{control_tab_pfn}
\alias{control_tab_pfn}
\title{Controlling TabPFN execution}
\usage{
control_tab_pfn(
  n_preprocessing_jobs = 1L,
  device = "auto",
  ignore_pretraining_limits = FALSE,
  inference_precision = "auto",
  fit_mode = "fit_preprocessors",
  memory_saving_mode = "auto",
  random_state = sample.int(10^6, 1),
  ...
)
}
\arguments{
\item{n_preprocessing_jobs}{An integer for the number of worker processes.
A value of -1L indicates all possible resources.}

\item{device}{A character value for the device used for torch (e.g., \code{"cpu"},
\code{"cuda"}, \code{"mps"}, etc.). Th default is \code{"auto"}.}

\item{ignore_pretraining_limits}{A logical to bypass the default data limits
on:the number of training set samples (10,000) and, the number of
predictors (500). There is an unchangeable limit to the number of classes
(10).}

\item{inference_precision}{A character value for the trade off between speed
and reproducibility. This can be a torch \code{dtype}, \code{"autocast"} (for torch's
mixed-precision autocast), or "auto".}

\item{fit_mode}{A character value to control how the are preprocessed and/or
cached. Values are \code{"fit_preprocessors"} (the default), \code{"low_memory"},
\code{"fit_with_cache"}, and \code{"batched"}.}

\item{memory_saving_mode}{A character string to help with out-of-memory
errors. Values are either a logical or \code{"auto"}.}

\item{random_state}{An integer to set the random number stream.}

\item{...}{Additional named arguments passed directly to the TabPFN Python
constructor. Use this to supply options not covered by the named parameters
above (e.g. arguments added in newer versions of the Python package).}
}
\value{
A list with extra class \code{"control_tab_pfn"} that has named elements
for each of the argument values.
}
\description{
Controlling TabPFN execution
}
\examples{
control_tab_pfn()
}
\references{
\url{https://github.com/PriorLabs/TabPFN/blob/main/src/tabpfn/classifier.py},
\url{https://github.com/PriorLabs/TabPFN/blob/main/src/tabpfn/regressor.py}
}
