% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tabpfn-package.R
\docType{package}
\name{tabpfn-package}
\alias{tabpfn}
\alias{tabpfn-package}
\title{tabpfn: Prior-Data Fitted Network Foundational Model for Tabular Data}
\description{
Provides a consistent API for classification and regression models based on the 'TabPFN' model of Hollmann et al. (2025), "Accurate predictions on small data with a tabular foundation model," Nature, 637(8045) \doi{10.1038/s41586-024-08328-6}. The calculations are served via 'Python' to train and predict the model.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://tabpfn.tidymodels.org}
  \item \url{https://github.com/tidymodels/tabpfn}
  \item Report bugs at \url{https://github.com/tidymodels/tabpfn/issues}
}

}
\author{
\strong{Maintainer}: Max Kuhn \email{max@posit.co} (\href{https://orcid.org/0000-0003-2402-136X}{ORCID})

Other contributors:
\itemize{
  \item Posit Software, PBC (\href{https://ror.org/03wc8by49}{ROR}) [copyright holder, funder]
}

}
\keyword{internal}
