% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tabpfn-predict.R
\name{predict.tab_pfn}
\alias{predict.tab_pfn}
\alias{augment.tab_pfn}
\title{Predict using \code{TabPFN}}
\usage{
\method{predict}{tab_pfn}(object, new_data, ...)

\method{augment}{tab_pfn}(x, new_data, ...)
}
\arguments{
\item{object, x}{A \code{tab_pfn} object.}

\item{new_data}{A data frame or matrix of new predictors.}

\item{...}{Not used, but required for extensibility.}
}
\value{
\code{\link[=predict]{predict()}} returns a tibble of predictions and \code{\link[=augment]{augment()}} appends the
columns in \code{new_data}. In either case, the number of rows in the tibble is
guaranteed to be the same as the number of rows in \code{new_data}.

For regression data, the prediction is in the column \code{.pred}. For
classification, the class predictions are in \code{.pred_class} and the
probability estimates are in columns with the pattern \verb{.pred_\{level\}} where
\code{level} is the levels of the outcome factor vector.
}
\description{
Predict using \code{TabPFN}
}
\examples{
# Minimal example for quick execution
car_train <- mtcars[ 1:5,   ]
car_test  <- mtcars[6, -1]

\dontrun{
# Fit
if (is_tab_pfn_installed() & interactive()) {
 mod <- tab_pfn(mpg ~ cyl + log(drat), car_train)

 # Predict
 predict(mod, car_test)
 augment(mod, car_test)
}
}

}
